package fi.jyu.intellij.comtest.rider.actions

import com.intellij.openapi.actionSystem.ActionManager
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rider.model.solutionAnalysisModel
import com.jetbrains.rider.projectView.solution
import comtest.v2.processors.ComTestResult
import fi.jyu.intellij.comtest.common.util.runWithRefresh
import fi.jyu.intellij.comtest.rider.ComTestRunner
import fi.jyu.intellij.comtest.rider.util.hasRiderSolution

/**
 * Action to run ComTest on the current .NET solution.
 */
@Suppress("MissingActionUpdateThread", "ComponentNotRegistered")
class ComTestGenerateDotNetSolutionAction : AnAction() {
    override fun actionPerformed(e: AnActionEvent) {
        val proj = e.project ?: return

        val res = runWithRefresh(proj) {
            ComTestRunner.generateForDotNetSln(proj)
        }

        if (res == null || (res !is ComTestResult.Success && res !is ComTestResult.SkippedAsGood)) {
            return
        }

        // csproj change will trigger assembly analysis
        // Wait for it to finish and then trigger unit test discovery
        val l = proj.lifetime.createNested()
        proj.solution.solutionAnalysisModel.ready.change.advise(l) {
            if (it) {
                l.terminate(true)
                val act = ActionManager.getInstance().getAction("RiderUnitTestRunSolutionAction")!!
                act.update(e)
                act.actionPerformed(e)
            }
        }
    }

    override fun update(e: AnActionEvent) {
        e.presentation.isEnabled = e.project?.hasRiderSolution ?: false
    }
}